.md-relbar2__inner{
    padding-top:0;
    padding-bottom:0;
    margin-bottom: 0;
}

.sphinx-tabs-tab {
  position: relative;
  font-family: "Chivo", "Colfax", Helvetica, Arial, sans-serif;
  color: #fb8c00;
  line-height: 24px;
  margin: 0;
  font-size: 19px;
  font-weight: 400;
  background-color: rgba(255, 255, 255, 0);
  border-radius: 5px 5px 0 0;
  border: 0;
  padding: 1rem 1.5rem;
  margin-bottom: 0;
}

.md-typeset{
    font-size:.7rem;
    line-height:1.6;
    -webkit-print-color-adjust:exact
}

body,input{
    color:rgba(0,0,0,.87);
    -webkit-font-feature-settings:"kern","liga";
    font-feature-settings:"kern","liga";
    font-family:"Chivo", "Colfax", "Franziska", Helvetica, Arial, sans-serif;
}

