CREATE SCHEMA IF NOT EXISTS test;
DROP TABLE IF EXISTS test.t1;
CREATE TABLE test.t1 (f1 INT) ENGINE=INNODB;
INSERT INTO test.t1 VALUES(1);
BEGIN;
UPDATE test.t1 SET f1=2;
SELECT SLEEP(11);
ROLLBACK;
DROP DATABASE IF EXISTS test;
