Create Table: CREATE TABLE `t` (
 `id` int(11) NOT NULL,
 `c` varchar(100) NOT NULL DEFAULT '' COMMENT 'Generated',
 `v` int(11) DEFAULT NULL,
 PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;
